<?php
/*
=== WordPress ===
Plugin Name: A8min
Plugin URI: http://hello.lumiere-couleur.com/a8min/
Description: A8.netの広告を管理し、サイズ別の画像を任意の場所へ表示します。
Version: 1.6
Author: smilkobuta
Author URI: http://hello.lumiere-couleur.com/smilkobuta/
=== WordPress ===
*/

/*######################################################################

    スタンドアロン実行を行うときには、下記の値を変更してください。

       $allow_standalone = true;

    その際はアクセス認証をかけるなどのセキュリティ対策を必ず行ってください。
    サーバー上のデータが漏洩する危険性があります！
*/

$allow_standalone = false;

/*######################################################################*/

$a8min_mode = 'basic';
if (file_exists(dirname(__FILE__) . '/include/code_mode.php')) {
    include('include/code_mode.php');
}

// WordPress実行チェック
$wpload = dirname(dirname(dirname(dirname(__FILE__)))) . "/wp-load.php";
if (file_exists($wpload)) {
    require_once($wpload);

    if ($_GET['ajax_ctl'] || $_POST['ajax_ctl']) {
        // Ajax呼び出し
        if (current_user_can('edit_plugins')) {
            $ajax_controller = true;
        }
    } else {
        // ログイン認証
        if (function_exists('auth_redirect')) {
            $_server = $_SERVER;
            $_SERVER['REQUEST_URI'] = preg_replace('/\/wp-content\/(.*)/', '/wp-admin/', $_SERVER['REQUEST_URI']);
            auth_redirect();
            $_SERVER = $_server;
        }
        if ($_GET['banner_select'] || $_POST['banner_select']) {
            $standalone = true;
        }
    }

    $pgtype = 'wp_plugin';

    $a8minurl =  get_option('siteurl') . '/wp-content/plugins/a8min';

    add_action('admin_menu', 'a8min_add_options_page');
    add_action('admin_head','a8min_options_panel_css');
    add_shortcode('a8min', 'a8min_shortcode');

    if ($a8min_mode == 'pro') {
        add_action('widgets_init', 'a8min_widget_register');
    	add_action('init', 'a8min_mceinit');
    	add_action('admin_print_scripts', 'a8min_mcescript');
        add_filter('the_content', 'a8min_autoinsert_content', 10);
        add_filter('the_excerpt', 'a8min_autoinsert_excerpt', 10);
        add_filter('the_content_rss', 'a8min_autoinsert_rss', 10);
        add_filter('the_excerpt_rss', 'a8min_autoinsert_rss', 10);
    }
}

// スタンドアロン実行用の設定
if ($pgtype != 'wp_plugin') {
    $pgtype = 'standalone';
    // 関数内で呼び出されたときのために、global宣言しておく。
    global $standalone, $sitedata_dir, $use_sqlite, $ajax_controller;
    global $a8min_defaults, $a8minurl, $a8min_settings, $a8min_mode;

    $a8minurl = $_SERVER['HTTPS'] ? 'https://' : 'http://'
        . $_SERVER['SERVER_NAME']
        . ':' . $_SERVER['SERVER_PORT']
        . dirname($_SERVER['REQUEST_URI']);
}

// スタンドアロン実行
if ($allow_standalone && basename($_SERVER['PHP_SELF']) == basename(__FILE__)) {
    $standalone = true;
    // Ajax呼び出し
    if ($_GET['ajax_ctl'] || $_POST['ajax_ctl']) {
        $ajax_controller = true;
    }
}

// ファイル＆ディレクトリ
$sitedata_dir = dirname(__FILE__) . '/sitedata';

// SQLite
$use_sqlite = function_exists('sqlite_open');

// デフォルト設定値
$a8min_defaults = array(
    'codedata_type' => 'text',
    'allow_duplicate' => false,
    'wide_minwidth_big' => 400,
    'wide_maxwidth_big' => -1,
    'wide_maxheight_big' => -1,
    'wide_minwidth_middle' => 200,
    'wide_maxwidth_middle' => 400,
    'wide_maxheight_middle' => -1,
    'wide_minwidth_small' => 40,
    'wide_maxwidth_small' => 200,
    'wide_maxheight_small' => -1,
    'long_minheight_big' => 400,
    'long_maxheight_big' => -1,
    'long_maxwidth_big' => -1,
    'long_minheight_middle' => 200,
    'long_maxheight_middle' => 400,
    'long_maxwidth_middle' => -1,
    'long_minheight_small' => 40,
    'long_maxheight_small' => 200,
    'long_maxwidth_small' => -1,
    'square_minwidth_big' => 400,
    'square_maxwidth_big' => -1,
    'square_minwidth_middle' => 200,
    'square_maxwidth_middle' => 400,
    'square_minwidth_small' => 40,
    'square_maxwidth_small' => 200,
    'wpoption_savedata_prefix_flg' => '0',
    'wpoption_savedata_prefix' => ''
);

$savedata_prefix = '';
$a8min_wp_settings = array();

if ($a8min_mode == 'pro') {
    include('include/code_savedata_prefix.php');
}

// 設定オプション取得
$a8min_settings = a8min_get_option();

if ($a8min_mode == 'pro') {
    include('include/code_link.php');
    include('include/code_autoinsert.php');
    include('include/code_editor_support.php');
    include('include/code_widget.php');
}

// バナー表示
function a8min($size, $option = '') {
    global $used_advs;
    global $a8min_settings;

    a8min_check_searchindex();

    $option['get_code'] = true;
    if ($size == 'text' || $size == 'mail') {
        $option['type'] = $size;
        $option['size'] = '';
    } else if (preg_match('/^s\d+_\d+$/', $size)) {
        $option['banner_id'] = $size;
    } else {
        $option['type'] = 'banner';
        $option['size'] = $size;
    }
    $advs = a8min_searchadv($option);

    if (!$a8min_settings['allow_duplicate']) {
        // 同じ広告を再表示しない
        if (!is_array($used_advs)) {
            $used_advs = array();
        }
        $filter = create_function('$adv', '
            global $used_advs;
            if (in_array($adv->banner_id, $used_advs)) {
                return false;
            } else {
                return true;
            }
        ');
        $advs = array_filter($advs, $filter);
    }

    if (count($advs) == 0) {
        return null;
    }

    $random = mt_rand(0, count($advs) - 1);
    $allkeys = array_keys($advs);
    $adv = $advs[$allkeys[$random]];
    $used_advs[] = $adv->banner_id;
    if ($option['return']) {
        return $adv->code;
    } else {
        print $adv->code;
    }
}

// 広告情報取得
function a8min_getadv($option = '') {
    $advs = a8min_get_sitedata('a8net', 'advs', $option);
    if (is_array($advs)) {
        if ($option['get_detail']) {
            $advsindex = a8min_get_searchindex($option);
            $advsattrib = a8min_get_advsattrib($option);
            $advs_new = array();
            foreach ($advs AS $k => $v) {
                $v->bannersize = $advsindex[$k]->bannersize;
                $v->category = $advsindex[$k]->category;
                $v->attrib = $advsattrib[$k];
                $advs_new[$k] = $v;
            }
            $advs = $advs_new;
        }
        return $advs;
    } else {
        return array();
    }
}

// プログラム詳細情報を取得
function a8min_getadvsinfo($option = '') {
    $advsinfo = a8min_get_sitedata('a8net', 'advsinfo', $option);
    if ($option['insId']) {
        return $advsinfo[$option['insId']];
    } else {
        return $advsinfo;
    }
}

// マッチするバナーを検索
function a8min_searchadv($option) {
    global $use_sqlite;

    $use_db = $use_sqlite && $option['get_code'];

    if (!$use_db) {
        // 詳細情報の取得
        if ($option['get_detail']) {
            $advs = a8min_getadv($option);
        } else {
            $advs = a8min_get_searchindex();
        }
    
        if ($option['banner_id']) {
            return array($option['banner_id'] => $advs[$option['banner_id']]);
        }
    }

    $size = $option['size'];
    $type = $option['type'];
    $merchant = $option['merchant'];
    $program_id = $option['program_id'];
    $program_category = $option['program_category'] ? $option['program_category'] : $option['category'];
    $program_tag = $option['program_tag'] ? $option['program_tag'] : $option['tag'];

    if ($program_category && !is_array($program_category)) {
        $program_category = array($program_category);
    } else if (!$program_category) {
        $program_category = array();
    }
    if ($program_tag != '' && !is_array($program_tag)) {
        $program_tag = array($program_tag);
    } else if (!$program_tag) {
        $program_tag = array();
    }

    if (preg_match('/(\d*)-(\d*)x(\d*)-(\d*)/', $size, $match)) {
        $size = array();
        $size['minwidth'] = $match[1];
        $size['maxwidth'] = $match[2];
        $size['minheight'] = $match[3];
        $size['maxheight'] = $match[4];
    }

    $ret = array();
    if ($use_db) {
        // DB検索
        $sqlite = a8min_sqlite();
        if (count($program_tag) > 0) {
            // タグIDを取得
            $sql_tags = "select tag_id, tag from adv_tag";
            $result = sqlite_array_query($sqlite, $sql_tags, SQLITE_ASSOC, $sqliteerror);
            $tag_ids = array();
            $tag_sql = array();
            if ($result) {
                foreach ($result AS $v) {
                    $tag_ids[] = $v['tag_id'];
                    $tag_names[] = $v['tag'];
                    $tag_sql[] = "tag_ids LIKE '%," . $v['tag_id'] . ",%'";
                }
            }
        }
        $tables[] = 'advs a';
        if ($tag_sql) {
            $where[] = '(' . join(' or ', $tag_sql) . ')';
        }
        if ($merchant) {
            $tables[] = 'merchant m';
            $where[] = 'm.merchant_id = a.merchant_id';
            $where[] = "m.merchant_name = '" . sqlite_escape_string($merchant) . "'";
        }
        if ($program_id) {
            $where[] = "a.program_id = '" . sqlite_escape_string($program_id) . "'";
        }
        if (count($program_category) > 0) {
            $tables[] = 'adv_category ac';
            $where[] = 'ac.category_id = a.category_id';
            $where_cat = array();
            foreach ($program_category AS $k => $v) {
                $where_cat[] = "ac.category_name = '" . sqlite_escape_string($v) . "'";
            }
            $where[] = '(' . join(' or ', $where_cat) . ')';
        }
        if ($option['banner_id']) {
            $where[] = "a.banner_id = '" . sqlite_escape_string($option['banner_id']) . "'";
        }
        if ($type) {
            $where[] = "a.type = '" . sqlite_escape_string($type) . "'";
        }
        if (is_array($size)) {
            if ($size['minwidth']) {
                $where[] = "a.banner_width >= " . intval($size['minwidth']);
            }
            if ($size['maxwidth']) {
                $where[] = "a.banner_width <= " . intval($size['maxwidth']);
            }
            if ($size['minheight']) {
                $where[] = "a.banner_height >= " . intval($size['minheight']);
            }
            if ($size['maxheight']) {
                $where[] = "a.banner_height <= " . intval($size['maxheight']);
            }
        } else if (preg_match('/([a-z]*)_([a-z]*)/', $size, $match)) {
            if ($match[1]) {
                $where[] = "a.banner_sizetype = '" . sqlite_escape_string($match[1]) . "'";
            }
            if ($match[2]) {
                $where[] = "a.banner_sizelevel = '" . sqlite_escape_string($match[2]) . "'";
            }
        }
        $sql = 'select a.banner_id as banner_id, a.code as code
                from ' . join(', ', $tables);
        if (count($where) > 0) {
            $sql .= ' where ' . join(' and ', $where);
        }
        $result = sqlite_array_query($sqlite, $sql, SQLITE_ASSOC, $sqliteerror);
        if ($sqliteerror) {
            print $sqliteerror;
        }
        if ($result) {
            foreach ($result AS $v) {
                $ret[$v['banner_id']]->banner_id = $v['banner_id'];
                $ret[$v['banner_id']]->code = $v['code'];
            }
        }
        sqlite_close($sqlite);
    } else {
        // ファイル検索
        foreach ($advs AS $k => $adv) {
            if (!$option['get_detail'] && $adv->attrib->notrecom) {
                continue;
            }
            if ((!$type || $adv->type == $type)
                    && (!$size || !$adv->bannersize || a8min_size_match($adv->bannersize, $size))
                    && (!$merchant || $adv->programAdvertiser == $merchant)
                    && (!$program_id || $adv->insId == $program_id)
                    && (!$program_category || array_search('', $program_category) !== false || in_array($adv->category, $program_category))
                    && (!$program_tag || array_search('', $program_tag) !== false || (is_array($adv->attrib->tag) && count(array_intersect($adv->attrib->tag, $program_tag)) > 0))) {
                $ret[$k] = $adv;
            }
        }
    }
    return $ret;
}

// バナー画像の種別（横長、たて長、スクエア）とサイズ（大中小）を決定
function a8min_getsize($banner_size, $a8min_settings) {
    global $a8min_defaults;
    $size_type = '';
    $size_level = 'out';
    if (preg_match('/(\d+) x (\d+)/', $banner_size, $match)) {
        $width = $match[1];
        $height = $match[2];
        if ($width / $height > 1.3) {
            // 横長
            $size_type = 'wide';
            if ($width >= $a8min_settings['wide_minwidth_big']
                    && $width <= $a8min_settings['wide_maxwidth_big']
                    && $height <= $a8min_settings['wide_maxheight_big']) {
                $size_level = 'big';
            } else if ($width >= $a8min_settings['wide_minwidth_middle']
                    && $width <= $a8min_settings['wide_maxwidth_middle']
                    && $height <= $a8min_settings['wide_maxheight_middle']) {
                $size_level = 'middle';
            } else if ($width >= $a8min_settings['wide_minwidth_small']
                    && $width <= $a8min_settings['wide_maxwidth_small']
                    && $height <= $a8min_settings['wide_maxheight_small']) {
                $size_level = 'small';
            }
        } else if ($height / $width > 1.3) {
            // たて長
            $size_type = 'long';
            if ($height >= $a8min_settings['long_minheight_big']
                    && $height <= $a8min_settings['long_maxheight_big']
                    && $width <= $a8min_settings['long_maxwidth_big']) {
                $size_level = 'big';
            } else if ($height >= $a8min_settings['long_minheight_middle']
                    && $height <= $a8min_settings['long_maxheight_middle']
                    && $width <= $a8min_settings['long_maxwidth_middle']) {
                $size_level = 'middle';
            } else if ($height >= $a8min_settings['long_minheight_small']
                    && $height <= $a8min_settings['long_minheight_small']
                    && $width <= $a8min_settings['long_maxwidth_small']) {
                $size_level = 'small';
            }
        } else {
            // スクエア
            $size_type = 'square';
            if ($width >= $a8min_settings['square_minwidth_big']
                    && $width <= $a8min_settings['square_maxwidth_big']) {
                $size_level = 'big';
            } else if ($width >= $a8min_settings['square_minwidth_middle']
                    && $width <= $a8min_settings['square_maxwidth_middle']) {
                $size_level = 'middle';
            } else if ($width >= $a8min_settings['square_minwidth_small']
                    && $width <= $a8min_settings['square_maxwidth_small']) {
                $size_level = 'small';
            }
        }
    }

    $ret = array(
        'width' => $width,
        'height' => $height,
        'size_type' => $size_type,
        'size_level' => $size_level
    );
    return $ret;
}

// 指定の画像種別とサイズに一致するかどうかをチェック
function a8min_size_match($banner_size, $expect_size) {
    $width = $banner_size['width'];
    $height = $banner_size['height'];
    $size_type = $banner_size['size_type'];
    $size_level = $banner_size['size_level'];
    if (is_array($expect_size)) {
        if ((!$expect_size['minwidth'] || $expect_size['minwidth'] <= $width)
                && (!$expect_size['maxwidth'] || $expect_size['maxwidth'] >= $width)
                && (!$expect_size['minheight'] || $expect_size['minheight'] <= $height)
                && (!$expect_size['maxheight'] || $expect_size['maxheight'] >= $height)) {
            return true;
        } else {
            return false;
        }
    } else {
        list($expect_size_type, $expect_size_level) = split('_', $expect_size);
        if ((!$expect_size_type || $expect_size_type == $size_type)
                && (!$expect_size_level || $expect_size_level == $size_level)) {
            return true;
        } else {
            return false;
        }
    }
}

// 保存されているサイトデータを取得
function a8min_get_sitedata($site, $key, $option = '') {
    global $sitedata_dir;
    global $savedata_prefix;
    static $ret;

    $cachekey = $site . '_' . $key;

    // キャッシュ削除
    if ($option['ignore_cache']) {
        unset($ret[$cachekey]);
    }

    // キャッシュ全削除
    if ($option['reset_cache']) {
    	$ret = array();
    }
    
    // キャッシュを返す
    if ($ret[$cachekey]) {
        return $ret[$cachekey];
    }

    if ($site == 'common') {
        // ローカルデータを参照
        $file = dirname(__FILE__) . '/sitedata/' . $savedata_prefix . $cachekey . '.txt';
    } else {
        // ローカルデータ／リンクデータを参照
        $file = $sitedata_dir . '/' . $savedata_prefix . $cachekey . '.txt';
    }
    if (file_exists($file)) {
        $ret[$cachekey] = unserialize(file_get_contents($file));
        return $ret[$cachekey];
    }
    return null;
}

// サイトデータを更新
function a8min_update_sitedata($site, $key, $sitedata) {
    global $sitedata_dir;
    global $savedata_prefix;
    
    $cachekey = $site . '_' . $key;
    
    if ($site == 'common') {
        // ローカルデータを更新
        $file = dirname(__FILE__) . '/sitedata/' . $savedata_prefix . $cachekey . '.txt';
    } else {
        // ローカルデータ／リンクデータを更新
        $file = $sitedata_dir . '/' . $savedata_prefix . $cachekey . '.txt';
    }
    $fp = fopen($file, 'w');
    if ($fp) {
        fwrite($fp, serialize($sitedata));
        fclose($fp);
        return true;
    }
    return false;
}

// 設定オプションを取得
function a8min_get_option($set_min_max = false, $option = '') {
    global $a8min_defaults;
    global $a8min_wp_settings;
    
    $ret = a8min_get_sitedata('common', 'option', $option);
    
    if (!$ret) {
    	$ret = array();
    }
    
    // 設定オプションとWordPress用の設定オプションをマージ
    $ret = array_merge($ret, $a8min_wp_settings);
    
    foreach ($a8min_defaults AS $k => $v) {
        $ret[$k] = ($ret[$k] || $ret[$k] !== '0') ? $ret[$k] : $v;
        // 最大値、最小値をセット
        if ($set_min_max && ($ret[$k] == '-1' || $ret[$k] === '')) {
            if (strpos($k, 'min')) {
                $ret[$k] = 0;
            } else if (strpos($k, 'max')) {
                $ret[$k] = PHP_INT_MAX;
            }
        }
    }
    return $ret;
}

// 設定オプションを更新
function a8min_update_option($value) {
	global $standalone;
	global $a8min_settings;
	
	if (!$standalone) {
	    foreach ($value as $k => $v) {
	        // 「wpoption_*」オプションは、WordPressのデータベースに格納する。
	        if (strpos($k, 'wpoption_') === 0) {
	            update_option('a8min_' . $k, $v);
	        }
	    }
	}
	
    $ret = a8min_update_sitedata('common', 'option', $value);
    
    // $a8min_settingsを更新
    if (function_exists('a8min_load_wpoption')) {
        a8min_load_wpoption();
    }
    $option = array('reset_cache' => true);
    $a8min_settings = a8min_get_option(true, $option);
    
    return $ret;
}

// 検索用のデータ配列を取得
function a8min_get_searchindex($option = '') {
    $ret = a8min_get_sitedata('common', 'search', $option);
    if (is_array($ret)) {
        return $ret;
    } else {
        return array();
    }
}

// 検索インデックスの更新チェック
function a8min_check_searchindex() {
	global $savedata_prefix;
    $attrib_mtime = @filemtime(dirname(__FILE__) . '/sitedata/' . $savedata_prefix . 'common_attrib.txt');
    $index_mtime = @filemtime(dirname(__FILE__) . '/sitedata/' . $savedata_prefix . 'common_search.txt');
    if (!$index_mtime || ($attrib_mtime && $attrib_mtime > $index_mtime)) {
        a8min_update_searchindex();
    }
}

// リンク検索インデックスの更新チェック
function a8min_check_searchindex_link() {
    global $sitedata_dir;
    $link_mtime = @filemtime($sitedata_dir . '/common_search.txt');
    $index_mtime = @filemtime(dirname(__FILE__) . '/sitedata/common_search.txt');
    if (!$index_mtime || ($link_mtime && $link_mtime > $index_mtime)) {
        a8min_update_searchindex();
    }
}

// 検索用のデータ配列を作成
function a8min_update_searchindex() {
    $option = array('ignore_cache' => true);
    $advs = a8min_getadv($option);
    $advsinfo = a8min_getadvsinfo($option);
    $advsattrib = a8min_get_advsattrib($option);
	$size_settings = a8min_get_option(true, $option);
    $keys = array_keys($advs);

    for ($i = 0; $i < count($keys); $i++) {
        $k = $keys[$i];
        if ($advs[$k]->type == 'banner') {
            $advs[$k]->bannersize = a8min_getsize($advs[$k]->size, $size_settings);
        }
        $advs[$k]->attrib = $advsattrib[$k];
        $advs[$k]->category = $advsinfo[$advs[$k]->insId]->category;
        unset($advs[$k]->image);
        unset($advs[$k]->size);
        unset($advs[$k]->banner_id);
        unset($advs[$k]->mediaId);
        unset($advs[$k]->programTitle);
    }

    a8min_update_sitedata('common', 'search', $advs);

    a8min_update_searchindex_db();
}

// SQLiteのinit
function a8min_sqlite() {
	global $savedata_prefix;
    $file = dirname(__FILE__) . '/sitedata/' . $savedata_prefix . 'common_search_v1.sqlite';
    $init_db = false;
    if (!file_exists($file)) {
        $init_db = true;
    }
    $sqlite = sqlite_open($file, 0666, $sqliteerror);
    if ($sqliteerror) {
        print $sqliteerror;
    } else if ($init_db) {
        // テーブル作成
        $sql = 'create table advs (
                    adv_id, banner_id, program_id, merchant_id, type, code
                    , banner_width, banner_height, banner_sizetype
                    , banner_sizelevel, category_id, tag_ids
                );
                create table merchant (
                    merchant_id, merchant_name
                );
                create table adv_category (
                    category_id, category_name
                );
                create table adv_tag (
                    tag_id, tag
                );
                ';
        $result = sqlite_exec($sqlite, $sql, $sqliteerror);
        if ($sqliteerror) {
            print $sqliteerror;
        }
    }
    return $sqlite;
}

// SQLiteデータを更新
function a8min_update_searchindex_db() {
    global $use_sqlite;
    if (!$use_sqlite) {
        return;
    }
    $option['ignore_cache'] = true;
    $advs = a8min_get_searchindex($option);
    
    $sqlite = a8min_sqlite();

    // データ準備
    $advs_data = array();
    $merchant = array();
    $category = array();
    $tag = array();

    foreach ($advs AS $k => $adv) {
        // 属性
        $adv->banner_id = $k;
        $adv->tag_ids = array();
        if ($adv->attrib) {
            if ($adv->attrib->notrecom) {
                // 非表示の広告は登録しない
                continue;
            } else if ($adv->attrib->tag) {
                // タグ情報
                foreach ($adv->attrib->tag AS $v) {
                    if (!in_array($v, $tag)) {
                        $tag[] = $v;
                    }
                    $adv->tag_ids[] = array_search($v, $tag) + 1;
                }
            }
        }
        // 広告主
        if ($adv->programAdvertiser) {
            if (!in_array($adv->programAdvertiser, $merchant)) {
                $merchant[] = $adv->programAdvertiser;
            }
            $adv->merchant_id = array_search($adv->programAdvertiser, $merchant) + 1;
        }
        // カテゴリー
        if ($adv->category) {
            if (!in_array($adv->category, $category)) {
                $category[] = $adv->category;
            }
            $adv->category_id = array_search($adv->category, $category) + 1;
        }
        $advs_data[] = $adv;
    }

    // 登録実行
    $sql = array();
    $sql[] = 'BEGIN TRANSACTION';
    $sql[] = 'delete from merchant';
    $sql[] = 'delete from adv_category';
    $sql[] = 'delete from adv_tag';
    $sql[] = 'delete from advs';
    foreach ($merchant AS $k => $v) {
        $merchant_id = $k + 1;
        $sql[] = 'insert into merchant (merchant_id, merchant_name)
                values (' . $merchant_id . ", '" . sqlite_escape_string($v) . "')";
    }
    foreach ($category AS $k => $v) {
        $category_id = $k + 1;
        $sql[] = 'insert into adv_category (category_id, category_name)
                values (' . $category_id . ", '" . sqlite_escape_string($v) . "')";
    }
    foreach ($tag AS $k => $v) {
        $tag_id = $k + 1;
        $sql[] = 'insert into adv_tag (tag_id, tag)
                values (' . $tag_id . ", '" . sqlite_escape_string($v) . "')";
    }
    foreach ($advs_data AS $k => $v) {
        $adv_id = $k + 1;
        $sql[] = 'insert into advs (adv_id, banner_id, program_id, merchant_id
                    , type, code, banner_width, banner_height, banner_sizetype
                    , banner_sizelevel, category_id, tag_ids)
                values (' . $adv_id . ", '" . sqlite_escape_string($v->banner_id) . "'
                , '" . sqlite_escape_string($v->insId) . "'
                , '" . sqlite_escape_string($v->merchant_id) . "'
                , '" . sqlite_escape_string($v->type) . "'
                , '" . sqlite_escape_string($v->code) . "'
                , '" . sqlite_escape_string($v->bannersize['width']) . "'
                , '" . sqlite_escape_string($v->bannersize['height']) . "'
                , '" . sqlite_escape_string($v->bannersize['size_type']) . "'
                , '" . sqlite_escape_string($v->bannersize['size_level']) . "'
                , '" . sqlite_escape_string($v->category_id) . "'
                , '," . sqlite_escape_string(join(',', $v->tag_ids)) . ",')";
    }
    $sql[] = 'END TRANSACTION';
    $sql[] = 'VACUUM';

    $result = sqlite_exec($sqlite, join(';', $sql), $sqliteerror);
    if ($sqliteerror) {
        print $sqliteerror;
    }

    sqlite_close($sqlite);
}

// バナーの付属情報を取得
function a8min_get_advsattrib($option = '') {
    $ret = a8min_get_sitedata('common', 'attrib', $option);
    if (!is_array($ret)) {
        $ret = array();
    }
    return $ret;
}

// バナーの付属情報を更新
function a8min_update_advsattrib($banner_id, $key, $value) {
    $option['ignore_cache'] = true;
    $advsattrib = a8min_get_advsattrib($option);
    $banner_id = split(',', $banner_id);
    foreach ($banner_id AS $v) {
        if ($value) {
            $advsattrib[$v]->$key = $value;
        } else {
            unset($advsattrib[$v]->$key);
        }
        if (count((array)$advsattrib[$v]) == 0) {
            unset($advsattrib[$v]);
        }
    }
    return a8min_update_sitedata('common', 'attrib', $advsattrib);
}

// バナーの付属情報をまとめて更新
function a8min_update_advsattrib_all($advsattrib) {
    return a8min_update_sitedata('common', 'attrib', $advsattrib);
}

// 存在しない広告の属性を削除
function a8min_gc_advsattrib() {
    $option['ignore_cache'] = true;
    $advsattrib = a8min_get_advsattrib($option);
    $advs = a8min_get_searchindex();
    $advsattrib_new = array();
    foreach ($advsattrib AS $k => $v) {
        if ($advs[$k]) {
            $advsattrib_new[$k] = $v;
        }
    }
    return a8min_update_sitedata('common', 'attrib', $advsattrib_new);
}

// コードデータを読み込んでパース
function a8min_parse_codedata($allcode) {
    $allcode = split("\n", $allcode);
    $advs = array();
    $advsinfo = array();

    $option = array('ignore_cache' => true);
	$size_settings = a8min_get_option(true, $option);
    if (count($allcode)) {
        $state_line = 'code';
        $current_insId = '';
        foreach ($allcode AS $line) {
            $line = trim($line);
            if (preg_match('/<\!-- a8min (.+) -->/', $line, $match)) {
                if ($obj) {
                    $advs[$obj->banner_id] = $obj;
                    $obj = '';
                }
                $attribs = split(',', $match[1]);
                $obj->insId = $attribs[0];
                $obj->programAdvertiser = $attribs[1];
                $obj->programTitle = $attribs[2];
                $obj->mediaId = $attribs[3];
                $obj->type = $attribs[4];
                $obj->size = $attribs[5];
                $obj->image = $attribs[6];
                $obj->banner_id = $attribs[7];
                $state_line = 'code';
            } else if (preg_match('/<\!-- a8min-ext (.+) -->/', $line, $match)) {
                $attribs = split(',', $match[1]);
                $current_insId = array_shift($attribs);
                if (count($attribs) == 1) {
                    $state_line = 'programInfo';
                } else {
                    $state_line = $attribs;
                }
            } else if ($line) {
                if ($state_line == 'code') {
                    $obj->code .= $line . "\n";
                } else if ($state_line == 'programInfo') {
                    $advsinfo[$current_insId]->programInfo .= $line . "\n";
                } else if (is_array($state_line) && count($state_line) > 0) {
                    $attrib_key = array_shift($state_line);
                    $advsinfo[$current_insId]->$attrib_key = $line;
                }
            }
        }
        if ($obj) {
            $advs[$obj->banner_id] = $obj;
        }
    }
    return array($advs, $advsinfo);
}

// 登録済みの広告タグを取得
function a8min_bannertag_list() {
    $advsattrib = a8min_get_advsattrib();
    $bannertags = array();
    foreach ($advsattrib AS $k => $v) {
        if ($v->tag) {
            foreach ($v->tag AS $tag) {
                $bannertags[$tag] = 1;
            }
        }
    }
    $bannertags = array_keys($bannertags);
    sort($bannertags);
    return $bannertags;
}

// 設定ページを登録
function a8min_add_options_page() {
	if (function_exists('add_options_page')) {
		add_options_page('A8min', 'A8min', 9, basename(__FILE__), 'a8min_options_panel');
	}
}

// 設定ページ用のCSS
function a8min_options_panel_css() {
    global $a8minurl;
?>
<link rel="stylesheet" href="<?= $a8minurl ?>/css/a8min.css" type="text/css" />
<?php
}

// 設定ページ
function a8min_options_panel() {
    global $a8minurl, $a8min_defaults, $pgtype, $a8min_settings;
    global $sitedata_dir, $use_sqlite, $a8min_mode;
    global $standalone;

    // バナー選択画面
    if ($_REQUEST['banner_select']) {
        a8min_banner_select();
        return;
    }

    $error = '';
    $getadv_option = '';
    $just_import = $_POST['just_import'];    // ダイレクトインポート

	if (isset($_POST['submitUpdate'])) {
	    if ($just_import) {
            // ダイレクトインポート
            if ($a8min_settings['codedata_type'] == 'link') {
                $error['codelink'] = 'リンク先のA8minのブックマークレットをご利用ください。';
                unset($_POST['allcode']);
            }
	    } else {
            // ダイレクトインポート以外はこちら
    		$a8min_settings['codelink'] = '';
            if ($_POST['codelink'] && $_POST['codedata_type'] == 'link') {
                if (is_dir($_POST['codelink']) && is_dir($_POST['codelink'] . '/sitedata')) {
            		$a8min_settings['codelink'] = $_POST['codelink'];
                    $sitedata_dir = $a8min_settings['codelink'] . '/sitedata';
                } else {
                    $error['codelink'] = '指定されたディレクトリにA8minを見つけられません。';
                }
            } else {
                $sitedata_dir = dirname(__FILE__) . '/sitedata';
            }
            // 未指定の項目にはデフォルト値を上書き
            foreach ($a8min_defaults AS $k => $v) {
                $a8min_settings[$k] = $_POST[$k] !== '' ? $_POST[$k] : $v;
            }
            // 設定オプションを更新
    		a8min_update_option($a8min_settings);
	    }

        // コードデータをパース実行＆更新
		$allcode = trim(stripslashes($_POST['allcode']));
        if ($just_import) {
            // 文字コード変換
            $allcode = mb_convert_encoding($allcode, 'UTF-8', 'EUC-JP,Shift_JIS,UTF-8,auto');
        }
        if ($allcode && $a8min_settings['codedata_type'] != 'link') {
            $advsdata = a8min_parse_codedata($allcode);
            a8min_update_sitedata('a8net', 'original', $allcode);
            a8min_update_sitedata('a8net', 'advs', $advsdata[0]);
            a8min_update_sitedata('a8net', 'advsinfo', $advsdata[1]);
            unset($_POST['allcode']);
        }

        // 存在しなくなった広告の属性情報を削除
        a8min_gc_advsattrib();

        // 検索インデックスの更新
        a8min_update_searchindex();
        $getadv_option = array('ignore_cache' => true);

	} else if ($_POST['submitAutoinsert']) {
	    // 自動挿入設定
	    $a8min_settings['autoinsert_post_before'] = trim(stripslashes($_POST['autoinsert_post_before']));
	    $a8min_settings['autoinsert_post_after'] = trim(stripslashes($_POST['autoinsert_post_after']));
	    $a8min_settings['autoinsert_excerpt_before'] = trim(stripslashes($_POST['autoinsert_excerpt_before']));
	    $a8min_settings['autoinsert_excerpt_after'] = trim(stripslashes($_POST['autoinsert_excerpt_after']));
	    $a8min_settings['autoinsert_rss_before'] = trim(stripslashes($_POST['autoinsert_rss_before']));
	    $a8min_settings['autoinsert_rss_after'] = trim(stripslashes($_POST['autoinsert_rss_after']));
		a8min_update_option($a8min_settings);
	} else if ($_POST['submitGeneral']) {
	    // 自動挿入設定
	    $a8min_settings['allow_duplicate'] = trim(stripslashes($_POST['allow_duplicate']));
		a8min_update_option($a8min_settings);
	}

    // 初回アクセス時
    if (!$just_import && !isset($a8min_settings['wide_minwidth_big'])) {
        foreach ($a8min_defaults AS $k => $v) {
        	if (!$a8min_settings[$k]) {
                $a8min_settings[$k] = $v;
        	}
        }
    }

    $advs = a8min_get_searchindex($getadv_option);

    $advs_cat = array();
    foreach ($advs AS $adv) {
        $advs_cat[$adv->type]['count']++;
        if ($adv->type == 'banner') {
            $advs_cat[$adv->type]['banner_type'][$adv->bannersize['size_type']][$adv->bannersize['size_level']]++;
        }
    }

    include('view/options_panel.php');
}

// バナー選択画面
function a8min_banner_select() {
    $dosearch = $_POST['dosearch'];
    $merchant = $_POST['merchant'];
    $program_id = $_POST['program_id'];
    $program_category = $_POST['program_category'];
    $program_tag = $_POST['program_tag'];
    $adv_type = $_POST['adv_type'];
    $adv_size_type = $_POST['adv_size_type'];
    $adv_size_level = $_POST['adv_size_level'];
    $disp_image = $_POST['disp_image'];

    if (!is_array($program_category)) {
        $program_category = array($program_category);
    }
    $program_category = array_filter($program_category);
    if ($program_tag) {
        $program_tag = array_map('trim', split(' ', $program_tag));
        $program_tag = array_filter($program_tag);
    } else {
        $program_tag = array();
    }
    if (!isset($adv_type) && !isset($disp_image)) {
        $disp_image = 1;
    }
    if (!isset($adv_type)) {
        $adv_type = 'banner';
    }

    // プログラム詳細情報の取得
    $advsinfo = a8min_getadvsinfo();
    $all_category = array();
    $all_merchant = array();
    $all_program = array();
    if (is_array($advsinfo)) {
        foreach ($advsinfo AS $k => $v) {
            $all_category[$v->category]++;
            $all_merchant[$v->programAdvertiser][] = $k;
            $all_program[$k] = $v->programTitle;
        }
    }
    ksort($all_category);
    ksort($all_merchant);
    asort($all_program);

    $all_tag = a8min_bannertag_list();

    // 検索実行
    if ($dosearch) {
        $size = $adv_size_type . '_' . $adv_size_level;
        $option = array(
            'get_detail' => true,
            'merchant' => $merchant,
            'program_id' => $program_id,
            'program_category' => $program_category,
            'program_tag' => $program_tag,
            'size' => $size,
            'type' => $adv_type
        );
        $advs = a8min_searchadv($option);

        // 貼り付け用コード
        $copycode_template = '<?= a8min("' . $size . '"';
        $copycode_shortcode = '[a8min size="' . $size . '"';
        $options_template = array();
        $options_shortcode = array();
        if (count($program_category) > 0) {
            $options_template[] = '"category"=>array("' . join('","', $program_category) . '")';
            $options_shortcode[] = 'category="' . join(',', $program_category) . '"';
        }
        if ($merchant) {
            $options_template[] = '"merchant"=>"' . $merchant . '"';
            $options_shortcode[] = 'merchant="' . $merchant . '"';
        }
        if ($program_id) {
            $options_template[] = '"program_id"=>"' . $program_id . '"';
            $options_shortcode[] = 'program_id="' . $program_id . '"';
        }
        if (count($program_tag) > 0) {
            $options_template[] = '"tag"=>array("' . join('","', $program_tag) . '")';
            $options_shortcode[] = 'tag="' . join(',', $program_tag) . '"';
        }
        if (count($options_template) > 0) {
            $copycode_template .= ',array(' . join(',', $options_template) . ')';
            $copycode_shortcode .= ' ' . join(' ', $options_shortcode);
        }
        $copycode_template .= ') ?>';
        $copycode_shortcode .= ']';
    }

    $a8min_typename = array(
        'banner' => 'バナー',
        'text' => 'テキスト',
        'mail' => 'メール'
    );
    include('view/banner_select.php');
}

// ショートコード
function a8min_shortcode($atts) {
    $option = array(
        'return' => 1,
        'merchant' => $atts['merchant'],
        'program_id' => $atts['program_id']
    );
    if ($atts['category']) {
        $option['category'] = split(',', $atts['category']);
    }
    if ($atts['tag']) {
        $option['tag'] = split(',', $atts['tag']);
    }
    return a8min($atts['id'] ? $atts['id'] : $atts['size'], $option);
}

// 直接呼出し
if ($_GET['ajax_ctl'] || $_POST['ajax_ctl']) {
    // Ajax呼び出し
    $callback = $_GET['callback'];
    $wordpress = ($pgtype == 'wp_plugin') ? '1' : '0';
    header('Content-Type: application/x-javascript');
    if ($ajax_controller) {
        print '(function(){' . $callback . '({ok:1,wordpress:' . $wordpress . '});})()';
    } else {
        // 念のためもう一回リダイレクトしてみる。
        if (!$_GET['redirect']) {
            $redirect = $_GET['redirect'] + 1;
            $url = $_SERVER['REQUEST_URI'] . '&redirect=' . $redirect;
            header("Location: $url");
            exit;
        }
        print '(function(){' . $callback . '({error:1,wordpress:' . $wordpress . ',url:"' . $url . '"});})()';
    }
} else if ($standalone) {
    // スタンドアロン実行
    require_once('view/base.php');
}
?>
